/*******************************************************************************
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.bpmn2bpel.components;

import java.util.List;
import org.ow2.orchestra.jaxb.bpmn.TEventBasedGateway;
import org.ow2.orchestra.jaxb.bpmn.TExclusiveGateway;
import org.ow2.orchestra.jaxb.bpmn.TFlowNode;
import org.ow2.orchestra.jaxb.bpmn.TParallelGateway;
import org.ow2.orchestra.jaxb.bpmn.TSequenceFlow;

/**
 * @author Guillaume Porcher
 */
public class OnEventComponent extends AbstractComponent {

  public enum Type {ACTIVITY, PARALLEL_FORK, PARALLEL_JOIN, EXCLUSIVE_FORK, EXCLUSIVE_JOIN, PICK};

  private final TFlowNode onEventNode;
  private final List<TFlowNode> pickOutNodes;
  private final List<TSequenceFlow> outSequenceFlows;
  private final List<TSequenceFlow> inSequenceFlows;

  public OnEventComponent(final TFlowNode onEventNode, final List<TFlowNode> pickOutNodes, final List<TSequenceFlow> outSequenceFlows, final List<TSequenceFlow> inSequenceFlows) {
    this.onEventNode = onEventNode;
    this.pickOutNodes = pickOutNodes;
    this.outSequenceFlows = outSequenceFlows;
    this.inSequenceFlows = inSequenceFlows;
  }

  public TFlowNode getOnEventNode() {
    return onEventNode;
  }

  public List<TSequenceFlow> getOutSequenceFlows() {
    return outSequenceFlows;
  }

  public List<TSequenceFlow> getInSequenceFlows() {
    return inSequenceFlows;
  }

  public List<TFlowNode> getPickOutNodes() {
    return pickOutNodes;
  }

  public Type getType() {
    if (onEventNode instanceof TParallelGateway) {
      if (outSequenceFlows.size() > 1) {
        return Type.PARALLEL_FORK;
      } else {
        return Type.PARALLEL_JOIN;
      }
    } else if (onEventNode instanceof TExclusiveGateway) {
      if (outSequenceFlows.size() > 1) {
        return Type.EXCLUSIVE_FORK;
      } else {
        return Type.EXCLUSIVE_JOIN;
      }
    } else if (onEventNode instanceof TEventBasedGateway) {
      return Type.PICK;
    }
    return Type.ACTIVITY;
  }
}
