/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel.generators;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import javax.wsdl.Definition;
import javax.wsdl.PortType;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import org.ow2.orchestra.bpmn2bpel.BpmnMap;
import org.ow2.orchestra.bpmn2bpel.JaxbUtil;
import org.ow2.orchestra.jaxb.b4p.PeopleActivity;
import org.ow2.orchestra.jaxb.b4p.TDocumentation;
import org.ow2.orchestra.jaxb.bpel.Assign;
import org.ow2.orchestra.jaxb.bpel.ExtensionActivity;
import org.ow2.orchestra.jaxb.bpel.FromPart;
import org.ow2.orchestra.jaxb.bpel.FromParts;
import org.ow2.orchestra.jaxb.bpel.Scope;
import org.ow2.orchestra.jaxb.bpel.Sequence;
import org.ow2.orchestra.jaxb.bpel.TActivity;
import org.ow2.orchestra.jaxb.bpel.ToPart;
import org.ow2.orchestra.jaxb.bpel.ToParts;
import org.ow2.orchestra.jaxb.bpel.Variable;
import org.ow2.orchestra.jaxb.bpel.Variables;
import org.ow2.orchestra.jaxb.bpmn.DataInputAssociation;
import org.ow2.orchestra.jaxb.bpmn.DataOutputAssociation;
import org.ow2.orchestra.jaxb.bpmn.Operation;
import org.ow2.orchestra.jaxb.bpmn.ResourceAssignmentExpression;
import org.ow2.orchestra.jaxb.bpmn.TPotentialOwner;
import org.ow2.orchestra.jaxb.bpmn.TResourceRole;
import org.ow2.orchestra.jaxb.bpmn.TUserTask;
import org.ow2.orchestra.jaxb.wsht.ObjectFactory;
import org.ow2.orchestra.jaxb.wsht.PeopleAssignments;
import org.ow2.orchestra.jaxb.wsht.Priority;
import org.ow2.orchestra.jaxb.wsht.TFrom;
import org.ow2.orchestra.jaxb.wsht.TGenericHumanRole;
import org.ow2.orchestra.jaxb.wsht.TPresentationElements;
import org.ow2.orchestra.jaxb.wsht.TTaskInterface;
import org.ow2.orchestra.jaxb.wsht.Task;
import org.xml.sax.SAXException;


/**
 * @author Guillaume Porcher
 */
class PeopleActivityGenerator {
  private final BpelGenerator bpelGenerator;
  private boolean addExtensionDeclaration = false;

  PeopleActivityGenerator() {
    this.bpelGenerator = new BpelGenerator();
  }

  /**
   * @param bpelGenerator
   */
  PeopleActivityGenerator(final BpelGenerator bpelGenerator) {
    this.bpelGenerator = bpelGenerator;
  }

  TActivity createPeopleActivity(final BpmnMap bpmnMap, final TUserTask userTask, final QName processQName, Map<String, Definition> partnerWsdls) throws JAXBException, IOException, SAXException {
    this.addExtensionDeclaration = true;
    final ExtensionActivity extensionActivity = new ExtensionActivity();

    final Assign inAssign = new Assign();
    final Assign outAssign = new Assign();
    final Scope scope = new Scope();
    final Variables variables = new Variables();
    scope.setVariables(variables);

    final PeopleActivity peopleActivity = new PeopleActivity();

    peopleActivity.setName(userTask.getId());
    final TDocumentation doc = new TDocumentation();
    doc.getContent().add(userTask.getName());
    peopleActivity.getDocumentations().add(doc);

    QName taskPortTypeQName = new QName(processQName.getNamespaceURI(), "taskPT" + userTask.getId(), "tns");
    PortType taskPortType = bpmnMap.getPortType(taskPortTypeQName);
    Operation taskOperation = new Operation();
    taskOperation.setName("taskOperation");


    OperationReference taskInOperationReference = new OperationReference(taskPortType, "...", taskOperation);
    // set fromParts
    final List<DataOutputAssociation> dataOutputAssociations = userTask.getDataOutputAssociations();
    if (!dataOutputAssociations.isEmpty()) {
      final Object dataObject =
        this.bpelGenerator.getOutputData(dataOutputAssociations, taskInOperationReference, outAssign, false, bpmnMap);
      if (dataObject instanceof FromParts) {
        org.ow2.orchestra.jaxb.b4p.FromParts b4pFromParts = new org.ow2.orchestra.jaxb.b4p.FromParts();
        for (FromPart toPart : ((FromParts) dataObject).getFromParts()) {
          org.ow2.orchestra.jaxb.b4p.FromPart b4pFromPart = new org.ow2.orchestra.jaxb.b4p.FromPart();
          b4pFromPart.setToVariable(toPart.getToVariable());
          b4pFromPart.setPart(toPart.getPart());
          b4pFromParts.getFromParts().add(b4pFromPart);
        }
        peopleActivity.setFromParts(b4pFromParts);
      } else if (dataObject instanceof Variable) {
        final Variable outVariable = (Variable) dataObject;
        scope.getVariables().getVariables().add(outVariable);
        peopleActivity.setOutputVariable(outVariable.getName());
      }
    }
    // set toParts
    final List<DataInputAssociation> dataInputAssociations = userTask.getDataInputAssociations();
    if (!dataInputAssociations.isEmpty()) {
      final Object dataObject =
        this.bpelGenerator.getInputData(dataInputAssociations, taskInOperationReference, inAssign, true, bpmnMap);
      if (dataObject instanceof ToParts) {
        org.ow2.orchestra.jaxb.b4p.ToParts b4pToParts = new org.ow2.orchestra.jaxb.b4p.ToParts();
        for (ToPart toPart : ((ToParts) dataObject).getToParts()) {
          org.ow2.orchestra.jaxb.b4p.ToPart b4pToPart = new org.ow2.orchestra.jaxb.b4p.ToPart();
          b4pToPart.setFromVariable(toPart.getFromVariable());
          b4pToPart.setPart(toPart.getPart());
          b4pToParts.getToParts().add(b4pToPart);
        }
        peopleActivity.setToParts(b4pToParts);
      } else if (dataObject instanceof Variable) {
        final Variable inVariable = (Variable) dataObject;
        scope.getVariables().getVariables().add(inVariable);
        peopleActivity.setInputVariable(inVariable.getName());
      }
    }


    // generate task:
    Task humanTask = new Task();
    humanTask.setName(userTask.getName());
    Priority priority = new Priority();
    priority.getContent().add("1");
    humanTask.setPriority(priority);
    PeopleAssignments peopleAssignments = new PeopleAssignments();
    humanTask.setPeopleAssignments(peopleAssignments);
    humanTask.setPresentationElements(new TPresentationElements());
    TTaskInterface taskInterface = new TTaskInterface();
    taskInterface.setPortType(taskPortTypeQName);
    taskInterface.setOperation(taskInOperationReference.getOperation().getName());
    humanTask.setInterface(taskInterface);

    for (JAXBElement<? extends TResourceRole> role : userTask.getResourceRoles()) {
        if (role.getDeclaredType().equals(TPotentialOwner.class)) {
          TPotentialOwner potentialOwner = (TPotentialOwner) role.getValue();
          ResourceAssignmentExpression potentialOwnerExpr = potentialOwner.getResourceAssignmentExpression();
          if (potentialOwnerExpr != null) {
            TGenericHumanRole potentialOwnerHumanRole = new TGenericHumanRole();
            TFrom potentialOwnerFrom = new TFrom();
            potentialOwnerFrom.getContent().addAll(bpelGenerator.getXPathExpression(potentialOwnerExpr.getExpression().getValue()));
            potentialOwnerFrom.getOtherAttributes().putAll(potentialOwnerExpr.getExpression().getValue().getOtherAttributes());

            potentialOwnerHumanRole.setFrom(potentialOwnerFrom);
            peopleAssignments.getPotentialOwnersAndExcludedOwnersAndTaskInitiators().add(new ObjectFactory().createPotentialOwners(potentialOwnerHumanRole));
          }
        }
      }

    peopleActivity.setTask(humanTask);
    // at the end serialize the peopleActivity
    extensionActivity.setAny(JaxbUtil.writePeopleActivity(peopleActivity));

    final Sequence sequence = new Sequence();
    if (!inAssign.getCopiesAndExtensionAssignOperations().isEmpty()) {
      sequence.getAssignsAndCompensatesAndCompensateScopes().add(inAssign);
    }
    sequence.getAssignsAndCompensatesAndCompensateScopes().add(extensionActivity);
    if (!outAssign.getCopiesAndExtensionAssignOperations().isEmpty()) {
      sequence.getAssignsAndCompensatesAndCompensateScopes().add(outAssign);
    }
    if (scope.getVariables().getVariables().isEmpty()) {
      return sequence;
    } else {
      scope.setSequence(sequence);
      return scope;
    }
  }

  public boolean isAddExtensionDeclaration() {
    return addExtensionDeclaration;
  }
}
