/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel.generators;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.bpmn2bpel.BpmnMap;
import org.ow2.orchestra.bpmn2bpel.exceptions.TransformerException;
import org.ow2.orchestra.jaxb.bpel.Assign;
import org.ow2.orchestra.jaxb.bpel.Documentation;
import org.ow2.orchestra.jaxb.bpel.FromParts;
import org.ow2.orchestra.jaxb.bpel.Receive;
import org.ow2.orchestra.jaxb.bpel.Scope;
import org.ow2.orchestra.jaxb.bpel.Sequence;
import org.ow2.orchestra.jaxb.bpel.TActivity;
import org.ow2.orchestra.jaxb.bpel.TBoolean;
import org.ow2.orchestra.jaxb.bpel.Variable;
import org.ow2.orchestra.jaxb.bpel.Variables;
import org.ow2.orchestra.jaxb.bpmn.DataOutputAssociation;
import org.ow2.orchestra.jaxb.bpmn.TCatchEvent;
import org.ow2.orchestra.jaxb.bpmn.TMessageEventDefinition;
import org.ow2.orchestra.jaxb.bpmn.TStartEvent;


/**
 * @author Guillaume Porcher
 */
class ReceiveGenerator {

  private final BpelGenerator bpelGenerator;

  ReceiveGenerator() {
    this.bpelGenerator = new BpelGenerator();
  }

  /**
   * @param bpelGenerator
   */
  ReceiveGenerator(final BpelGenerator bpelGenerator) {
    this.bpelGenerator = bpelGenerator;
  }

  TActivity createReceiveActivity(final BpmnMap bpmnMap, final QName processQName,
      final TCatchEvent event, final TMessageEventDefinition messageEventDefinition) {
    final Receive receive = new Receive();

    receive.setName(event.getId());
    final Documentation doc = new Documentation();
    doc.getContent().add(event.getName());
    receive.getDocumentations().add(doc);

    final Assign assign = new Assign();
    assign.setName(event.getId() + "_dataOutput");
    final Documentation outAssignDoc = new Documentation();
    outAssignDoc.getContent().add("DataOutput assignment for " + event.getName());
    assign.getDocumentations().add(outAssignDoc);

    final Scope scope = new Scope();
    scope.setName(event.getId() + "_scope");
    final Documentation scopeDoc = new Documentation();
    scopeDoc.getContent().add("Scope for " + event.getName());
    scope.getDocumentations().add(scopeDoc);

    final Variables variables = new Variables();
    scope.setVariables(variables);


    if (event instanceof TStartEvent) {
      receive.setCreateInstance(TBoolean.YES);
    } else {
      receive.setCreateInstance(TBoolean.NO);
    }

    if (messageEventDefinition.getOperationRef() == null) {
      throw new TransformerException("message events must define an operation.");
    }

    final QName operationQName = messageEventDefinition.getOperationRef();


    final OperationReference receiveOperation =
      this.bpelGenerator.getProcessOperationReference(bpmnMap, processQName, operationQName);
    if (receiveOperation == null) {
      throw new TransformerException("operation not found");
    }
    receive.setOperation(receiveOperation.getOperation().getName());
    receive.setPartnerLink(receiveOperation.getPartnerLinkName());
    receive.setPortType(receiveOperation.getPortTypeQName());

    // set fromParts
    final List<DataOutputAssociation> dataOutputAssociations = event.getDataOutputAssociations();
    if (!dataOutputAssociations.isEmpty()) {
      final Object dataObject =
        this.bpelGenerator.getOutputData(dataOutputAssociations, receiveOperation, assign, true, bpmnMap);
      if (dataObject instanceof FromParts) {
        receive.setFromParts((FromParts) dataObject);
      } else if (dataObject instanceof Variable) {
        final Variable outVariable = (Variable) dataObject;
        scope.getVariables().getVariables().add(outVariable);
        receive.setVariable(outVariable.getName());
      }
    }

    if (assign.getCopiesAndExtensionAssignOperations().isEmpty()) {
      return receive;
    } else {
      final Sequence sequence = new Sequence();
      sequence.setName(event.getId() + "_sequence");
      final Documentation sequenceDoc = new Documentation();
      sequenceDoc.getContent().add("Sequence for " + event.getName());
      sequence.getDocumentations().add(sequenceDoc);
      sequence.getAssignsAndCompensatesAndCompensateScopes().add(receive);
      sequence.getAssignsAndCompensatesAndCompensateScopes().add(assign);
      if (scope.getVariables().getVariables().isEmpty()) {
        return sequence;
      } else {
        scope.setSequence(sequence);
        return scope;
      }
    }
  }

}
