/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.bpmn2bpel.generators;

import javax.wsdl.Definition;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import org.ow2.orchestra.bpmn2bpel.BpelUtil;
import org.ow2.orchestra.bpmn2bpel.BpmnMap;
import org.ow2.orchestra.bpmn2bpel.components.RepeatComponent;
import org.ow2.orchestra.bpmn2bpel.components.RepeatWhileComponent;
import org.ow2.orchestra.bpmn2bpel.components.WhileComponent;
import org.ow2.orchestra.jaxb.bpel.Condition;
import org.ow2.orchestra.jaxb.bpel.Sequence;
import org.ow2.orchestra.jaxb.bpel.TActivity;
import org.ow2.orchestra.jaxb.bpel.While;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.util.Map;


/**
 * @author Guillaume Porcher
 */
class WhileGenerator {
  private final BpelGenerator bpelGenerator;

  WhileGenerator() {
    this.bpelGenerator = new BpelGenerator();
  }

  /**
   * @param bpelGenerator
   */
  WhileGenerator(final BpelGenerator bpelGenerator) {
    this.bpelGenerator = bpelGenerator;
  }

  TActivity createWhileActivity(
          final BpmnMap bpmnMap,
          final WhileComponent whileComponent,
          final QName processQName,
          final Map<String, Definition> partnerWsdls) throws JAXBException, IOException, SAXException {
    // create bpel while
    final While whileActivity = new While();

    final TActivity activity = this.bpelGenerator.createBpelActivity(whileComponent.getWhileNode(), bpmnMap, processQName, partnerWsdls);
    BpelUtil.setChildActivity(whileActivity, activity);

    final Condition whileCondition = new Condition();
    whileCondition.getContent().addAll(this.bpelGenerator.getXPathExpression(whileComponent.getCondition()));
    whileCondition.getOtherAttributes().putAll(whileComponent.getCondition().getOtherAttributes());
    whileActivity.setCondition(whileCondition);

    return whileActivity;
  }

  TActivity createRepeatActivity(
          final BpmnMap bpmnMap,
          final RepeatComponent repeatComponent,
          final QName processQName,
          Map<String, Definition> partnerWsdls) throws JAXBException, IOException, SAXException {
    // create sequence
    final Sequence sequence = new Sequence();

    // add first activity execution
    final TActivity activity = this.bpelGenerator.createBpelActivity(repeatComponent.getFlowNode(), bpmnMap, processQName, partnerWsdls);
    sequence.getAssignsAndCompensatesAndCompensateScopes().add(activity);

    // create bpel while
    final While whileActivity = new While();

    BpelUtil.setChildActivity(whileActivity, activity);

    final Condition whileCondition = new Condition();
    whileCondition.getContent().addAll(this.bpelGenerator.getXPathExpression(repeatComponent.getCondition()));
    whileCondition.getOtherAttributes().putAll(repeatComponent.getCondition().getOtherAttributes());

    whileActivity.setCondition(whileCondition);

    // add while activity to sequence
    sequence.getAssignsAndCompensatesAndCompensateScopes().add(whileActivity);

    return sequence;
  }

  TActivity createRepeatWhileActivity(
          final BpmnMap bpmnMap,
          final RepeatWhileComponent repeatComponent,
          final QName processQName,
          Map<String, Definition> partnerWsdls) throws JAXBException, IOException, SAXException {
    // create sequence
    final Sequence sequence = new Sequence();

    // add first activity execution
    final TActivity repeatActivity =
            this.bpelGenerator.createBpelActivity(repeatComponent.getRepeatNode(), bpmnMap, processQName, partnerWsdls);
    sequence.getAssignsAndCompensatesAndCompensateScopes().add(repeatActivity);

    // create bpel while
    final While whileActivity = new While();

    // create whileActivity
    final TActivity whileNodeActivity =
            this.bpelGenerator.createBpelActivity(repeatComponent.getWhileNode(), bpmnMap, processQName, partnerWsdls);
    final Sequence whileSequence = new Sequence();
    whileSequence.getAssignsAndCompensatesAndCompensateScopes().add(whileNodeActivity);
    whileSequence.getAssignsAndCompensatesAndCompensateScopes().add(repeatActivity);

    whileActivity.setSequence(whileSequence);

    final Condition whileCondition = new Condition();
    whileCondition.getContent().addAll(this.bpelGenerator.getXPathExpression(repeatComponent.getCondition()));
    whileCondition.getOtherAttributes().putAll(repeatComponent.getCondition().getOtherAttributes());
    whileActivity.setCondition(whileCondition);

    // add while activity to sequence
    sequence.getAssignsAndCompensatesAndCompensateScopes().add(whileActivity);

    return sequence;
  }

}
